from time import time as get_systime_sec

from edge.webserver.sysconf import ph
from edge.model.geolocation import Geolocation


class TzServiceArgs:
    def __init__(self, tz_info):
        """
        :type tz_info: datetime.tzinfo
        """
        self.tz_info = tz_info


class GpsdServiceArgs:
    PROVIDER_STATIC = 'static'
    PROVIDER_GPSD = 'gpsd'
    _MAX_ENTRY_COUNT = 100
    _MAX_AGE_DELTA_USEC = int(60 * 1e6)
    _MAX_LOOKUP_DIST_USEC = int(5 * 1e6)

    def __init__(self, provider=PROVIDER_STATIC, geodata=None, endpoint=None, azimuth=None,
                 max_entry_count=_MAX_ENTRY_COUNT,
                 max_age_delta_usec=_MAX_AGE_DELTA_USEC,
                 max_lookup_dist_usec=_MAX_LOOKUP_DIST_USEC):
        """
        :type provider: str
        :type geodata: Geolocation | None
        :type endpoint: (str, int) | None
        :type azimuth: float | None
        :type max_entry_count: int
        :type max_age_delta_usec: int
        :type max_lookup_dist_usec: int
        """
        self.provider = provider
        self.azimuth = azimuth
        self.geodata = geodata
        self.endpoint = endpoint
        self.max_entry_count = max_entry_count
        self.max_age_delta_usec = max_age_delta_usec
        self.max_lookup_dist_usec = max_lookup_dist_usec


class GeodataCounters:
    """
    :type mts: long | None
    :type tpv_ts: long | None
    :type tpv_lat: float | None
    :type tpv_lon: float | None
    :type tpv_alt: float | None
    """
    def __init__(self):
        self.mts = None

        self.tpv_ts = None
        self.tpv_lat = None
        self.tpv_lon = None
        self.tpv_alt = None

    def inc(self, tpv_ts, tpv_lat, tpv_lon, tpv_alt):
        """
        :type tpv_ts: long | None
        :type tpv_lat: float | None
        :type tpv_lon: float | None
        :type tpv_alt: float | None
        """
        self.mts = int(get_systime_sec())

        self.tpv_ts = tpv_ts
        self.tpv_lat = tpv_lat
        self.tpv_lon = tpv_lon
        self.tpv_alt = tpv_alt


class GpsdCounters:
    """
    :type mode: int
    :type mode_mts: int
    :type store_count: long
    """

    MODE_0D_FIX = 0
    MODE_1D_FIX = 1
    MODE_2D_FIX = 2
    MODE_3D_FIX = 3

    def __init__(self):
        self.mode = 0
        self.mode_mts = 0

        self.store_count = long()
        self.geodata = GeodataCounters()

    def inc_mode(self, mode):
        if self.mode != mode:
            self.mode = mode
            self.mode_mts = int(get_systime_sec())

    def inc_store_count(self, count):
        self.store_count += count


class AboutServiceArgs:
    def __init__(self, origin_about_path, server_about_path=ph.CTRL_UICONF_PATH):
        """
        :type origin_about_path: str
        :type server_about_path: str
        """
        self.origin_about_path = origin_about_path
        self.server_about_path = server_about_path
